proc GetInfoProj { index } {
# Scan an Existing Project :
#	o checking for sub-directories,
#	o list of Programs (executable or defined),
#	o list of Targets,
#	o list of Bugged Targets.
#
	global Obj_Proj Obj_Dir Glob_Pref CheckLevel

	set proj $Obj_Proj($index)

	if [ info exists Obj_Proj($index) ] {
		show-status "Get infos for Project $proj" proj-status
	} else {
		show-status "No Project with index : $index" main-status
		return
	}

	set projpath	$Obj_Dir(Proj)/$proj
	set Src		$projpath/Src
	set Builder	$projpath/Builder
	set Pgm		$projpath/Pgm

	set Fdat	$projpath/Fdat
	set Fres	$projpath/Fres

	# Check for Project sub-directories ...
	foreach rep "$Src $Builder $Pgm $Fdat $Fres" {
		if ![file isdirectory $rep] {
			makedir $rep
		}
	}

	# Programs :
	# List of Executables ...
	set Obj_Proj($index,list_prg) {}

	foreach file [cd $Pgm ; glob -nocomplain *] {
		if [file executable $file] {
			lappend Obj_Proj($index,list_prg) $file
		}
	}
	# ... And List of .def
	foreach file [cd  $Builder  ; glob -nocomplain *.def] {
		regsub "\.def$" $file {} prg
		set Obj_Proj($index,list_prg) \
			[AppendItemInList $Obj_Proj($index,list_prg) $prg]
	}
	set Obj_Proj($index,list_prg) [lsort -ascii $Obj_Proj($index,list_prg)]

	# Targets
	set Obj_Proj($index,list_regf) \
		[cd $Src ; glob -nocomplain *.$Glob_Pref(FortranExt)]
	set Obj_Proj($index,list_regf) [lsort -ascii $Obj_Proj($index,list_regf)]

	set Obj_Proj($index,nb_regf) [ llength  $Obj_Proj($index,list_regf) ]

	# Bugged Targets : get the {*.errcp} list
	set Obj_Proj($index,list_errcp) \
		[cd $Src ; glob -nocomplain *.errcp]
	set Obj_Proj($index,list_errcp) [lsort -ascii $Obj_Proj($index,list_errcp)]

	set Obj_Proj($index,nb_errcp) [ llength  $Obj_Proj($index,list_errcp) ]

	# Programs Infos and Status
	# I disabled that option baceause it is too much Time consuming
	# so, checking program should be done explicitly when needeed...
	# See the new proc GetInfoPrg
#	GetInfoAllPrg $index $proj

	show-status "Ready..." proj-status
}
proc GetInfoAllPrg {index proj} {
# Programs Infos and Status

	global Obj_Proj

	set Obj_Proj($index,list_rebp) {}
	foreach prg $Obj_Proj($index,list_prg) {

		# Program Infos
		GetInfoPrg $index $proj $prg

		# Include Program in the list ?
		if { $Obj_Proj($index,$prg,status) != {} } {
			lappend Obj_Proj($index,list_rebp) $prg
		}
	}
	# End

	set Obj_Proj($index,nb_rebp) [ llength  $Obj_Proj($index,list_rebp) ]

	if { $Obj_Proj($index,nb_rebp) > 0 } {
		show-status "$Obj_Proj($index,nb_rebp) Program(s) To Update" proj-status
	} else {
		CheckLevelStatus prg
		if {$CheckLevel(prg,on)} {
			show-status "Nothing To Do !" proj-status
		} else {
			show-status "Checking not fully active !" proj-status
		}
	}
}
proc GetInfoPrg {index proj prg} {
# Get Infos on Program prg from Project proj (#index in list)
	global Obj_Dir Obj_Proj Obj_WinProj Glob_Pref Obj_Libar Obj_Lib CheckLevel Process

	set projpath $Obj_Dir(Proj)/$proj
	set Src		$projpath/Src
	set Builder	$projpath/Builder
	set Pgm		$projpath/Pgm

	# Definitions
	set Obj_Proj($index,$prg,target)	[GetPrgDef $proj $prg SrcF77u]
	set Obj_Proj($index,$prg,libusr)	[GetPrgDef $proj $prg LibF77u]
	set Obj_Proj($index,$prg,libcom)	[GetPrgDef $proj $prg LibF77c]

	# Status
	set Obj_Proj($index,$prg,target_reb)	{}
	set Obj_Proj($index,$prg,status)	{}

	# Default status for targets is "safe" (i.e. 0)
	foreach target $Obj_Proj($index,list_regf) {
		set Obj_Proj($index,$prg,$target) 0
	}

	set trace_type proj
	set trace_id $index:$prg
	set Process($trace_type,checking,$trace_id) {start checking}

	show-status "Start Checking ... " $Obj_WinProj($index,wstatus)
	PRINT "Checking Program $prg from Project $proj"

	# Build a list of Programs for which
	# (re-)Building should be done, because :

	set flag_check 0

# Level 1	# The Program is Defined but not Executable :
		# -----------------------------------------

	incr flag_check
        if {$CheckLevel(prg,$flag_check,flag)} {
		set Process($trace_type,checking,$trace_id) "Check level #${flag_check}"
		set iflag 0
		if ![file executable $Pgm/$prg] { incr iflag }
		if {$iflag} {
		lappend Obj_Proj($index,$prg,status) \
				"Check Level #$flag_check :\nExecutable Doesn't Exists !"
		}
	}

# Level 2	# The Program has a linking error :
		# -------------------------------

	incr flag_check
        if {$CheckLevel(prg,$flag_check,flag)} {
	set Process($trace_type,checking,$trace_id) "Check level #${flag_check}"
	set iflag 0
	if [file exists $Builder/$prg.errlnk] { incr iflag }
		if {$iflag} {
			lappend Obj_Proj($index,$prg,status) \
				"Check Level #$flag_check:\nLinking Error(s) !"
			lappend Obj_Proj($index,$prg,status) [ReadInFile $Builder/$prg.errlnk]
		}
	}


# Level 3	# The Program had a linking warnings :
		# ----------------------------------

	incr flag_check
        if {$CheckLevel(prg,$flag_check,flag)} {
		set Process($trace_type,checking,$trace_id) "Check level #${flag_check}"
		set iflag 0
		if [file exists $Builder/$prg.warnlnk] { incr iflag }
		if {$iflag} {
			lappend Obj_Proj($index,$prg,status) \
				"Check Level #$flag_check:\nLinking Warning(s) !"
			lappend Obj_Proj($index,$prg,status) [ReadInFile $Builder/$prg.warnlnk]
		}
	}

# Level 4	# Some Targets of Program are still bugged :
		# ----------------------------------------

	incr flag_check
        if {$CheckLevel(prg,$flag_check,flag)} {
		set Process($trace_type,checking,$trace_id) "Check level #${flag_check}"
		set iflag 0 ; set checklist {}
		foreach fileerrcp $Obj_Proj($index,list_errcp) {
			# the bugged file is a prog's target ?
			regsub "\.errcp$" $fileerrcp .$Glob_Pref(FortranExt) target
			if [ ItemIsInList $target $Obj_Proj($index,$prg,target)] {
			 	set Obj_Proj($index,$prg,target_reb) \
					[AppendItemInList $Obj_Proj($index,$prg,target_reb) $target]
				incr iflag
				lappend checklist "Target: $target"
				set Obj_Proj($index,$prg,$target) 1
			}
		}
		if {$iflag} {
			lappend Obj_Proj($index,$prg,status) \
				"Check Level #$flag_check :\n$iflag Bugged Target(s) !"
			lappend Obj_Proj($index,$prg,status) $checklist
		}
	}

# Level 5	# Some Targets are newer that the Program :
		# ---------------------------------------

	incr flag_check
        if {$CheckLevel(prg,$flag_check,flag)} {
		set Process($trace_type,checking,$trace_id) "Check level #${flag_check}"
		set iflag 0 ; set checklist {}
		set fileref $Pgm/$prg
		foreach target $Obj_Proj($index,$prg,target) {
			if [newer $Src/$target $fileref] {
			 	set Obj_Proj($index,$prg,target_reb) \
					[AppendItemInList $Obj_Proj($index,$prg,target_reb) $target]
				incr iflag
				lappend checklist "Target: $target"
				set Obj_Proj($index,$prg,$target) 1
			}
		}
		if {$iflag} {
			lappend Obj_Proj($index,$prg,status) \
				"Check Level #$flag_check :\n$iflag Newer Target(s) vs Program !"
			lappend Obj_Proj($index,$prg,status) $checklist
		}
	}

# Level 6	# Some Targets have no Object Module
		# ----------------------------------

	incr flag_check
        if {$CheckLevel(prg,$flag_check,flag)} {
		set Process($trace_type,checking,$trace_id) "Check level #${flag_check}"
		set iflag 0 ; set checklist {}
		foreach target $Obj_Proj($index,$prg,target) {
			regsub "\.$Glob_Pref(FortranExt)$" $target {.o} fileobj
			if ![file exists $Builder/$fileobj] {
			 	set Obj_Proj($index,$prg,target_reb) \
					[AppendItemInList $Obj_Proj($index,$prg,target_reb) $target]
				incr iflag
				lappend checklist "Target: $target"
				set Obj_Proj($index,$prg,$target) 1
			}
		}
		if {$iflag} {
			lappend Obj_Proj($index,$prg,status) \
				"Check Level #$flag_check :\n$iflag Target(s) without Object Module"
			lappend Obj_Proj($index,$prg,status) $checklist
		}
	}

# Level 7	# Some Targets are newer than their Object Module :
		# -----------------------------------------------

	incr flag_check
        if {$CheckLevel(prg,$flag_check,flag)} {
		set Process($trace_type,checking,$trace_id) "Check level #${flag_check}"
		set iflag 0 ; set checklist {}
		foreach target $Obj_Proj($index,$prg,target) {
			regsub "\.$Glob_Pref(FortranExt)$" $target {.o} fileobj
			set fileref $Builder/$fileobj
			if [file exists $fileref] {
				if [newer $Src/$target $fileref] {
			 		set Obj_Proj($index,$prg,target_reb) \
						[AppendItemInList $Obj_Proj($index,$prg,target_reb) $target]
					incr iflag
					lappend checklist "Target: $target"
					set Obj_Proj($index,$prg,$target) 1
				}
			}
		}
		if {$iflag} {
			lappend Obj_Proj($index,$prg,status) \
				"Check Level #$flag_check :\n$iflag Newer Target(s) vs Object Modules"
			lappend Obj_Proj($index,$prg,status) $checklist
		}
	}

# Level 8	# Some User's Archive Libraries are newer that the Program :
		# --------------------------------------------------------

	incr flag_check
        if {$CheckLevel(prg,$flag_check,flag)} {
		set Process($trace_type,checking,$trace_id) "Check level #${flag_check}"
		set iflag 0 ; set checklist {}
		set fileref $Pgm/$prg
		foreach libusr $Obj_Proj($index,$prg,libusr) {
			if [newer $Obj_Dir(Lib)/Lib.ar/$libusr $fileref] {
				incr iflag
				lappend checklist "User's Lib: $libusr"
			}
		}
		if {$iflag} {
			lappend Obj_Proj($index,$prg,status) \
				"Check Level #$flag_check :\n$iflag Newer User's Lib(s) vs Program !"
			lappend Obj_Proj($index,$prg,status) $checklist
		}
	}

# Level 9	# Some Shared's Archive Libraries are newer that the Program :
		# ----------------------------------------------------------

	incr flag_check
        if {$CheckLevel(prg,$flag_check,flag)} {
		set Process($trace_type,checking,$trace_id) "Check level #${flag_check}"
		set iflag 0 ; set checklist {}
		set fileref $Pgm/$prg
		foreach libcom $Obj_Proj($index,$prg,libcom) {
			if [newer $Glob_Pref(SharedLib)/$libcom $fileref] {
				incr iflag
				lappend checklist "Shared Lib: $libcom"
			}
		}
		if {$iflag} {
			lappend Obj_Proj($index,$prg,status) \
				"Check Level #$flag_check :\n$iflag Newer Shared Lib(s) vs Program !"
			lappend Obj_Proj($index,$prg,status) $checklist
		}
	}

# Level 10	# Some Members of Libraries need (re-)building :
		# --------------------------------------------

	incr flag_check
        if {$CheckLevel(prg,$flag_check,flag)} {
		set Process($trace_type,checking,$trace_id) "Check level #${flag_check}"
		foreach libusr $Obj_Proj($index,$prg,libusr) {
			if [ info exists Obj_Libar($libusr,libindex) ] {
				set libindex $Obj_Libar($libusr,libindex)
				set lib $Obj_Lib($libindex)
				set libstat [GetInfoLib $libindex]
				if { $libstat != {} } {
					lappend Obj_Proj($index,$prg,status) \
						"Check Level #$flag_check :\nWarning on Library ..."
					lappend Obj_Proj($index,$prg,status) "$libstat"
					
				}
			}
		}
	}

	set Process($trace_type,checking,$trace_id) {end checking}

	# Inform the user
	# through the Project status widget

	if { $Obj_Proj($index,$prg,status) == {} } {
		CheckLevelStatus prg
		if {$CheckLevel(prg,on)} {
			show-status "Program is Up To Date" $Obj_WinProj($index,wstatus)
		} else {
			show-status "Checking not fully active !"  $Obj_WinProj($index,wstatus)
		}
	} else {
			show-status "Program Should Be Updated !"  $Obj_WinProj($index,wstatus)
	}

}
proc SavePrgDef {numproj prg}  {
	global Obj_Dir Obj_Proj
	set proj $Obj_Proj($numproj)
	set file $Obj_Dir(Proj)/$proj/Builder/$prg.def

	# Open a stream for file to save
	if [catch {open $file w} fileId] {
		PRINT "proc SavePrgDef : $fileId"
		tk_dialog .d$numproj {TkfPW Error} "Cannot write in file\n$file" info 0 Ok
		return 0
	} else {
		# Write the Program Definitions
		puts $fileId "# TkfPW file. Do not edit please."
		foreach target $Obj_Proj($numproj,$prg,target) {
			puts $fileId "SrcF77u:$target"
		}
		foreach libusr $Obj_Proj($numproj,$prg,libusr) {
			puts $fileId "LibF77u:$libusr"
		}
		foreach libcom $Obj_Proj($numproj,$prg,libcom) {
			puts $fileId "LibF77c:$libcom"
		}
		close $fileId
		return 1
	}
}
proc GetPrgDef {proj prg keyword} {
# Return the constitutive list of infos for Program prg of Project proj
# matching the keyword definition from file ".def" in Builder
	global Obj_Dir
	set filedef $Obj_Dir(Proj)/$proj/Builder/$prg.def
	set listdef {}
	if ![catch {open $filedef r} fileId] {
		while { [gets $fileId line] >=0 } {
			# Check for Targets
			if [ regexp (^$keyword) $line ] {
				foreach item [lrange [split $line :] 1 end] {
					lappend listdef [string trim $item]
				}
			}
		}
		close $fileId
	}
	return $listdef
}
